<?php
/*
Plugin Name: Purehelp.se Widget
Plugin URI: http://purehelp.se
Description: This is a plugin which fetches company information from purehelp.se and presents it in a widget. Use [purehelp_se org_number="123456-1234"] shortcode; custom_field org_number_se or write the organization number in content.

Version: 0.1
Author: Purehelp
Author URI: http://purehelp.se
License: MIT
*/
/*
Copyright (c) 2013 Purehelp AS

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

// Don't expose any info if called directly
if ( !function_exists( 'add_action' ) ) {
	echo 'Purehelp.se widget should not be called directly.';
	exit;
}

global $args, $instance;

// extend WP_Widget class
class Purehelp_se_Widget extends WP_Widget {
	public $ph_attr=array(
		'org_number' => null,
		'api_key' => '',
		'options' => array(
			'title' => 'Företagsinfo',
			'show_graph' => 1,
			'show_search' => 1,
			'hide_sidebar' => 0,
			'from_shortcode' =>0,
		),
	);

	/* constructor */
	function __construct() {
		parent::WP_Widget( /* Base ID */'purehelp_se',/* Name */'Purehelp.se', array( 'description' =>'En widget för att visa företagsinformation från purehelp.se'));
		
		// Set default api_key
		if (($this->ph_attr['api_key']=get_option('widget_purehelp_se_apikey'))===false) {
			$api_src=(string) get_option('siteurl');
			$api_src.=(string) get_option('blogname');
			$api_src.=(string) time();
			$this->ph_attr['api_key']=md5($api_src);
			add_option('widget_purehelp_se_apikey', $this->ph_attr['api_key']);
		}
	}

	/** @see WP_Widget::widget */
	public function widget( $args, $instance ) {

		// Don't waste time
		if (!$this->ph_attr['options']['from_shortcode'] && isset($instance['hide_sidebar']) && $instance['hide_sidebar']) {
			return;
		}
		
		global $wpdb, $post;

	 	//	Init 
		$this->ph_attr['options'] = wp_parse_args( (array) $instance, $this->ph_attr['options'] );
	 	$orgNumber=&$this->ph_attr['org_number'];
	 	$phContent=null;

	 	// 	Have you created a custom_field: "org_number_se" for the post?
	 	if (!$orgNumber) {
	 		$cf = get_post_custom();
	 		if (!empty($cf['org_number_se'][0]))
	 			$orgNumber=$cf['org_number_se'][0];
	 	}
	 	
		// 	Is it part of the content and properly formatted 123456-1234 ?
		if (!$orgNumber) {
			$content = strip_shortcodes(get_the_content()); 
			if (preg_match('/(?<![0-9])([0-9]{6}-[0-9]{4})(?![0-9])/', $content, $match)) {
				$orgNumber=$match[1];
			}
		}

		// Make sure it's numeric
		$orgNumber=preg_replace('/[^0-9]/','',$orgNumber);

		// Missing orgnumber, then return quietly
		if (!is_numeric($orgNumber))
			return;

		extract($args);

		$title = apply_filters( 'widget_title', $instance['title'] );


		// add an array to add the mime types to after we check which user options have been selected.
		$mime_type_array = array();
		if (!($api_key=$this->ph_attr['api_key']))
			$api_key='none';
		
		// Fetch data & Cache
		if ( ( $phData = get_transient( 'phse_'.$orgNumber ) ) === false ) {
			$url='http://www.purehelp.se/export/widget/'.$api_key.'/'.$orgNumber.'/';

			$req=wp_remote_get( $url );
			$phContent=wp_remote_retrieve_body($req);

			// Parse json
			$phData=json_decode($phContent,true);

			// Set cache 12 hours - even for empty results
			if (!empty($phContent))
		 		set_transient( 'phse_'.$orgNumber, $phData, 43200);
		}

		// Display content
		if ( !empty($phData) ) {
			echo $before_widget,
				'<section class="purehelp_se_widget">',
				$before_title, 
				$title, 
				$after_title,
				(!empty($phData['t1'])?$phData['t1']:''),
				(!empty($phData['t2'])?$phData['t2']:''),
				(($this->ph_attr['options']['show_graph']) && !empty($phData['g'])?$phData['g']:''),
				(($this->ph_attr['options']['show_search']) && !empty($phData['s'])?$phData['s']:''),
				'</section>',
				$after_widget;
		}

	}

	/** @see WP_Widget::update */
	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		$new_instance = wp_parse_args( (array) $new_instance, array( 
			'title' => '',
			'api_key' => '',
			'show_graph' => 0,
			'show_search' => 0,
			'hide_sidebar' => 0) 
		);

		$instance['title'] = strip_tags($new_instance['title']);
		$instance['api_key'] = strip_tags($new_instance['api_key']);
		$instance['show_graph'] = $new_instance['show_graph'] ? 1 : 0;		
		$instance['show_search'] = $new_instance['show_search'] ? 1 : 0;
		$instance['hide_sidebar'] = $new_instance['hide_sidebar'] ? 1 : 0;

		if ($instance['api_key']) {
			if (!($myApiKey=get_option('widget_purehelp_se_apikey'))) {
				add_option('widget_purehelp_se_apikey', $instance['api_key']);
			} else {
				if ($myApiKey!=$instance['api_key']) {
					update_option('widget_purehelp_se_apikey', $instance['api_key']);
				}
			}
		}
		// Don't store this value local
		unset ($instance['api_key']);

		return $instance;
	}

	/** @see WP_Widget::form */
	function form( $instance=array() ) {

		$instance = wp_parse_args( (array) $instance, array( 
			'title' => 'Företagsinfo',
			'api_key' => get_option('widget_purehelp_se_apikey'),
			'show_graph' => 1,
			'show_search' => 1,
			'hide_sidebar' => 0) 
		);

		$title = esc_attr($instance['title']);
		$api_key = get_option('widget_purehelp_se_apikey');

		echo '<p><label for="'.$this->get_field_id('title').'">'._e('Title:').'</label><input class="widefat" id="'.$this->get_field_id('title').'" name="'.$this->get_field_name('title').'" type="text" value="'.$title.'" /></p>';
		
		echo '<p><label for="'.$this->get_field_id('api_key').'">API nyckel:</label> 
		<input class="widefat" id="'.$this->get_field_id('api_key').'" name="'.$this->get_field_name('api_key').'" type="text" value="'.$api_key.'" /></p>
		
		<p><em>Välj innehåll att visa:</em><br />
		<input class="checkbox" type="checkbox" '.checked($instance['show_graph'], 1,0).' id="'.$this->get_field_id('show_graph').'" name="'.$this->get_field_name('show_graph').'" /> <label for="'.$this->get_field_name('show_graph').'">Visa graf</label><br />		
		<input class="checkbox" type="checkbox" '.checked($instance['show_search'],1,0).' id="'.$this->get_field_id('show_search').'" name="'.$this->get_field_name('show_search').'" /> <label for="'.$this->get_field_name('show_search').'">Visa sök</label><br />';

		echo '<hr />		
		<p><em>Dölj widgeten från sidofältet om du använder kortkod:</em><br />
		<input class="checkbox" type="checkbox" '.checked($instance['hide_sidebar'],1,0).' id="'.$this->get_field_id('hide_sidebar').'" name="'.$this->get_field_name('hide_sidebar').'" /> <label for="'.$this->get_field_name('hide_sidebar').'">Dölj från sidofältet</label></p>';

	}

} 

function purehelp_se($attr=array()) {
	// Don't waste time when nothing todo
	if (empty($attr['org_number']))
		return;

	// Get the array values of the widget so we can pass it back to the widget class function.
	$instance = array_shift(array_values(get_option('widget_purehelp_se')));

	$args = array(
		'before_widget' => '<div class="widget-wrapper widget_purehelp_se">',
		'after_widget' => '</div>',
		'before_title' => '<div class="widget-title">',
		'after_title' => '</div>'
	);

	// Add a new widget class object
	$PHSEWidget = new Purehelp_se_Widget();
	$PHSEWidget->ph_attr['org_number']=$attr['org_number'];
	$PHSEWidget->ph_attr['options']['from_shortcode']=1;
	
	ob_start();
	$PHSEWidget->widget($args,$instance);
	$out=ob_get_contents();
	ob_end_clean();
	return $out;
}

// Add stylesheet
function purehelp_se_css() {
	wp_enqueue_style('purehelp_se_css', plugins_url('css/purehelp_se_widget.css', __FILE__));
}

add_action( 'wp_enqueue_scripts', 'purehelp_se_css');

// register widget
function purehelp_se_register_widget() {
	register_widget( 'Purehelp_se_Widget' );
}
add_action( 'widgets_init', 'purehelp_se_register_widget' );

// Add the shortcode action
add_shortcode('purehelp_se','purehelp_se'); 

?>